luautil

package module
v0.1.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Aug 5, 2026 License: MIT Imports: 3 Imported by: 0

README

LuaUtil

LuaUtil is a small library written in go that provides the toolset to work with lua code in a structured manner.

For contributors

To update the yacc stuff you need goyacc.

go get modernc.org/goyacc

Command to generate the go file from yacc:

goyacc -o parser.go parser.y

You can delete the y.output file afterwards.

Sources

The parser and ast is forked from gopher-lua and somewhat modified.

The ssa implementation is inspired by the GoLang ssa package.

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Quote

func Quote(s string) string

Quote returns a double-quoted lua string literal representing s. The returned string uses lua escape sequences (\t, \n, \123, \u0100) for control characters and non-printable characters as defined by IsPrint.

Types

This section is empty.

Directories

Path Synopsis

Jump to